require 'spec_helper'
include RedirHelper

describe :redir do

  after :each do
    Redirect.delete_all
    RedirectLog.delete_all
  end

  shared_examples_for 'code that redirects' do

    before do
      before
      get redirect.shortened_path.to_s
    end

    context 'redirects' do
      before do
        follow_redirect!
      end

      context 'url' do
        subject {last_request}
        its(:url) {should eq redirect.original_url}
      end

      context 'response' do
        subject {last_response.errors.size}
        it {should eq 0}
      end
    end

  end

  let(:before) {Time.now}
  let(:after) {Time.now}

  context 'should apply class level settings for the url and path' do
    before do
      SecureRandom.should_receive(:urlsafe_base64).with(6).and_return('eeeeee')
      Redirect.path = 'foo/'
      Redirect.host = 'http://myredir.com/'
    end

    let(:redirect) { Redirect.create_shortened_url 'http://www.google.com/' }
    subject {redirect}
    its (:shortened_id) {should eq 'eeeeee'}
    its (:shortened_path) {should eq 'foo/eeeeee'}
    its (:shortened_url) {should eq 'http://myredir.com/foo/eeeeee'}

    context 'non default path' do
      it_behaves_like 'code that redirects'
    end

    after do
      Redirect.path = 'a/'
      Redirect.host = 'http://localhost:3000/'
    end
  end


  context 'general redirect' do

    let(:redirect) {create :redirect}
    before do
      before
      get redirect.shortened_path.to_s
    end

    it_behaves_like 'code that redirects'

    let(:redirect_log) {RedirectLog.where(:redirect_id => redirect.id).to_a.last}

    context 'headers' do
      subject {redirect_log.headers}
      its (['HTTP_HOST']) {should eq 'example.org'}
      its (['HTTP_COOKIE']) {should eq ''}
    end

    context 'request time' do
      subject {redirect_log}
      its (:request_time) {should > before}
      its (:request_time) {should < after}
    end 
  end

  context 'redirect with tracking info appended' do
    before do
      Redirect.append_tracking_info = true 
      get redirect.shortened_path.to_s
      follow_redirect!
    end
    let (:redirect_log) {Redirector.find_logs(redirect.shortened_id).last}

    context 'with existing url data' do
      let (:redirect) {create :redirect, :original_url => 'www.google.com/?foo=bar'}
      subject {last_request}
      its (:url) {should eq "http://www.google.com/?foo=bar&redirect_mongo_id=#{redirect_log.redirect_id}" +
                            "&redirect_log_mongo_id=#{redirect_log._id}"}
    end

    context 'without existing url data' do
      let (:redirect) {create :redirect, :original_url => 'www.google.com'}
      subject {last_request}
      its (:url) {should eq "http://www.google.com/?redirect_mongo_id=#{redirect_log.redirect_id}" +
                            "&redirect_log_mongo_id=#{redirect_log._id}"}
    end

    after do
      Redirect.append_tracking_info = false
    end
  end

  context 'redirect with google analytics info appended' do
    before do
      Redirect.append_google_analytics = true 
      get redirect.shortened_path.to_s
      follow_redirect!
    end
    let (:redirect_log) {Redirector.find_logs(redirect.shortened_id).last}

    context 'with existing url data' do
      let (:redirect) {create :redirect, :original_url => 'www.google.com/?foo=bar'}
      subject {last_request}
      its (:url) {should eq "http://www.google.com/?foo=bar&utm_source=http%3A%2F%2Flocalhost%3A3000%2F" +
                            "&utm_campaign=#{redirect_log.redirect_id}" +
                            "&utm_medium=referral"}
    end

    context 'without existing url data' do
      let (:redirect) {create :redirect, :original_url => 'www.google.com'}
      subject {last_request}
      its (:url) {should eq "http://www.google.com/?utm_source=http%3A%2F%2Flocalhost%3A3000%2F" +
                            "&utm_campaign=#{redirect_log.redirect_id}" +
                            "&utm_medium=referral"}
    end

    after do
      Redirect.append_google_analytics = false
    end
  end

  context 'redirect with no preceeding http' do
    before do
      get redirect.shortened_path.to_s
      follow_redirect!
    end

    context 'url without http' do
      let (:redirect) {create :redirect, :original_url => 'www.google.com'}
      subject {last_request}
      its (:url) {should eq 'http://www.google.com/'}
    end

    context 'url without http and leading space' do
      let (:redirect) {create :redirect, :original_url => ' www.google.com'}
      subject {last_request}
      its (:url) {should eq 'http://www.google.com/'}
    end
  end

  context 'bad redirect' do

    before do
      get "/a/ugh"
    end

    context 'status' do
      subject {last_response.status}
      it {should eq 404}
    end

  end

  context 'non rdr path' do

    before do
      get "/rddd/ugh"
    end

    context 'status' do
      subject {last_response.status}
      it {should eq 200}
    end

  end

end


